/**********************************************************************
 *
 * querytools.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * PUT COPYRIGHT NOTICE HERE
 *
 * $Id: querytools.cpp 349 1999-07-07 06:13:10Z rjmcnab $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.2  1999/07/07 06:12:21  rjmcnab
   Added ability to combine two independant queries.

   Revision 1.1  1999/06/15 02:24:06  sjboddie
   moved do_query from queryaction to new querytools module (as do_action
   is also called from documentaction when highlighting text)

 */


#include "querytools.h"


// do_query sets the filter options and makes the protocol call to
// do a query. The results are returned in response. 
// request.filterResultOptions and request.fields (if required)
// should be set by the calling function.
// If the query string uses quotes (phrase-searching) the quoted
// part of the string is returned in quotedstring (as something
// of a side-effect ;-/

bool do_query (FilterRequest_t &request, cgiargsclass &args, 
	       recptproto *collectproto, text_t &quotedstring, 
	       FilterResponse_t &response, ostream &logout) {

  request.filterName = "QueryFilter";

  comerror_t err;
  OptionValue_t option;
  text_t formattedstring = args["q"];

  // format the query string ready to do the query (and remember
  // the quoted part for later)
  format_querystring (formattedstring, quotedstring);
  
  option.name = "Term";
  option.value = formattedstring;
  request.filterOptions.push_back (option);

  option.name = "QueryType";
  option.value = (args.getintarg("t")) ? "ranked" : "boolean";
  request.filterOptions.push_back (option);

  option.name = "Casefold";
  option.value = (args.getintarg("k")) ? "true" : "false";
  request.filterOptions.push_back (option);

  option.name = "Stem";
  option.value = (args.getintarg("s")) ? "true" : "false";
  request.filterOptions.push_back (option);

  if (!args["h"].empty()) {
    option.name = "Index";
    option.value = args["h"];
    request.filterOptions.push_back (option);
  }

  if (!args["j"].empty()) {
    option.name = "Subcollection";
    option.value = args["j"];
    request.filterOptions.push_back (option);
  }

  if (!args["n"].empty()) {
    option.name = "Language";
    option.value = args["n"];
    request.filterOptions.push_back (option);
  }

  // fill in the second query if needed
  if (!args["cq2"].empty()) {
    option.name = "CombineQuery";
    option.value = args["cq2"];
    request.filterOptions.push_back (option);
    
    text_t quotedstring2;
    text_t formattedstring2 = args["q2"];
    format_querystring (formattedstring2, quotedstring2);

    option.name = "Term";
    option.value = formattedstring2;
    request.filterOptions.push_back (option);

    option.name = "QueryType";
    option.value = (args.getintarg("t")) ? "ranked" : "boolean";
    request.filterOptions.push_back (option);

    option.name = "Casefold";
    option.value = (args.getintarg("k")) ? "true" : "false";
    request.filterOptions.push_back (option);

    option.name = "Stem";
    option.value = (args.getintarg("s")) ? "true" : "false";
    request.filterOptions.push_back (option);

    if (!args["h2"].empty()) {
      option.name = "Index";
      option.value = args["h2"];
      request.filterOptions.push_back (option);
    }

    if (!args["j2"].empty()) {
      option.name = "Subcollection";
      option.value = args["j2"];
      request.filterOptions.push_back (option);
    }

    if (!args["n2"].empty()) {
      option.name = "Language";
      option.value = args["n2"];
      request.filterOptions.push_back (option);
    }
  }
  
  option.name = "StartResults";
  option.value = args["r"];
  request.filterOptions.push_back (option);

  option.name = "EndResults";
  int arg_m = args.getintarg("m");
  int endresults = args.getintarg("o") + (args.getintarg("r") - 1);
  if (endresults > arg_m) endresults = arg_m;
  option.value = endresults;
  request.filterOptions.push_back (option);

  collectproto->filter (args["c"], request, response, err, logout);

  if (err != noError) {
    outconvertclass text_t2ascii;
    logout << text_t2ascii
	   << "Error: call to QueryFilter failed in queryaction ("
	   << get_comerror_string (err) << ")\n";
    return false;
  }
  return true;
}

void format_querystring (text_t &querystring, text_t &quotedstring) {
  text_t formattedstring;
  quotedstring.clear();

  text_t::const_iterator here = querystring.begin();
  text_t::const_iterator end = querystring.end();
  int foundquote = 0;

  // space is used to insert spaces between Chinese
  // characters. No space is needed before the first
  // Chinese character.
  bool space = false;

  // want to remove ()|!& from querystring so boolean queries are just
  // "all the words" queries
  while (here != end) {
    if (*here == '(' || *here == ')' || *here == '|' ||
	*here == '!' || *here == '&') {
      formattedstring.push_back(' ');
    } else {
      if (*here == '"') {
	if (foundquote) {foundquote = 0; quotedstring.push_back(*here);}
	else foundquote = 1;
      } else {
	if ((*here >= 0x4e00 && *here <= 0x9fa5) ||
	    (*here >= 0xf900 && *here <= 0xfa2d)) {
	  // Chinese character
	  if (space) formattedstring.push_back (0x200b);
	  formattedstring.push_back (*here);
	  formattedstring.push_back (0x200b);
	  space = true;
	} else {
	  // non-Chinese character
	  formattedstring.push_back (*here);
	  space = false;
	}
      }
      if (foundquote) quotedstring.push_back(*here);
    }
    here ++;
  }
  querystring = formattedstring + quotedstring;
}

